@@ -1,4 +1,4 @@
-from __future__ import absolute_import
+import typing
 
 import six as _six
 from flyteidl.admin import common_pb2 as _common_pb2
@@ -19,13 +19,26 @@ from flytekit.models import execution as _execution
 from flytekit.models import filters as _filters
 from flytekit.models import launch_plan as _launch_plan
 from flytekit.models import node_execution as _node_execution
+from flytekit.models import project as _project
 from flytekit.models import task as _task
+from flytekit.models.admin import common as _admin_common
 from flytekit.models.admin import task_execution as _task_execution
 from flytekit.models.admin import workflow as _workflow
 from flytekit.models.core import identifier as _identifier
 
 
 class SynchronousFlyteClient(_RawSynchronousFlyteClient):
+    """
+    This is a low-level client that users can use to make direct gRPC service calls to the control plane. See the
+    :std:doc:`service spec <idl:protos/docs/service/index>`. This is more user-friendly interface than the
+    :py:class:`raw client <flytekit.clients.raw.RawSynchronousFlyteClient>` so users should try to use this class
+    first. Create a client by ::
+
+        SynchronousFlyteClient("your.domain:port", insecure=True)
+        # insecure should be True if your flyteadmin deployment doesn't have SSL enabled
+
+    """
+
     @property
     def raw(self):
         """
@@ -317,7 +330,8 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         """
         super(SynchronousFlyteClient, self).create_launch_plan(
             _launch_plan_pb2.LaunchPlanCreateRequest(
-                id=launch_plan_identifer.to_flyte_idl(), spec=launch_plan_spec.to_flyte_idl(),
+                id=launch_plan_identifer.to_flyte_idl(),
+                spec=launch_plan_spec.to_flyte_idl(),
             )
         )
 
@@ -507,7 +521,9 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         """
         super(SynchronousFlyteClient, self).update_named_entity(
             _common_pb2.NamedEntityUpdateRequest(
-                resource_type=resource_type, id=id.to_flyte_idl(), metadata=metadata.to_flyte_idl(),
+                resource_type=resource_type,
+                id=id.to_flyte_idl(),
+                metadata=metadata.to_flyte_idl(),
             )
         )
 
@@ -662,17 +678,24 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
 
     def list_node_executions(
-        self, workflow_execution_identifier, limit=100, token=None, filters=None, sort_by=None,
+        self,
+        workflow_execution_identifier,
+        limit: int = 100,
+        token: typing.Optional[str] = None,
+        filters: typing.List[_filters.Filter] = None,
+        sort_by: _admin_common.Sort = None,
+        unique_parent_id: str = None,
     ):
-        """
-        TODO: Comment
+        """Get node executions associated with a given workflow execution.
+
         :param flytekit.models.core.identifier.WorkflowExecutionIdentifier workflow_execution_identifier:
-        :param int limit:
-        :param Text token: [Optional] If specified, this specifies where in the rows of results to skip before reading.
-        If you previously retrieved a page response with token="foo" and you want the next page,
-        specify token="foo".
+        :param limit: Limit the number of items returned in the response.
+        :param token: If specified, this specifies where in the rows of results to skip before reading.
+            If you previously retrieved a page response with token="foo" and you want the next page,
+            specify ``token="foo"``.
         :param list[flytekit.models.filters.Filter] filters:
         :param flytekit.models.admin.common.Sort sort_by: [Optional] If provided, the results will be sorted.
+        :param unique_parent_id: If specified, returns the node executions for the ``unique_parent_id`` node id.
         :rtype: list[flytekit.models.node_execution.NodeExecution], Text
         """
         exec_list = super(SynchronousFlyteClient, self).list_node_executions_paginated(
@@ -682,6 +705,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
                 token=token,
                 filters=_filters.FilterList(filters or []).to_flyte_idl(),
                 sort_by=None if sort_by is None else sort_by.to_flyte_idl(),
+                unique_parent_id=unique_parent_id,
             )
         )
         return (
@@ -690,7 +714,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
 
     def list_node_executions_for_task_paginated(
-        self, task_execution_identifier, limit=100, token=None, filters=None, sort_by=None,
+        self,
+        task_execution_identifier,
+        limit=100,
+        token=None,
+        filters=None,
+        sort_by=None,
     ):
         """
         This returns nodes spawned by a specific task execution.  This is generally from things like dynamic tasks.
@@ -748,7 +777,12 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         )
 
     def list_task_executions_paginated(
-        self, node_execution_identifier, limit=100, token=None, filters=None, sort_by=None,
+        self,
+        node_execution_identifier,
+        limit=100,
+        token=None,
+        filters=None,
+        sort_by=None,
     ):
         """
         :param flytekit.models.core.identifier.NodeExecutionIdentifier node_execution_identifier:
@@ -787,7 +821,56 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         :rtype: flyteidl.admin.project_pb2.ProjectRegisterResponse
         """
         super(SynchronousFlyteClient, self).register_project(
-            _project_pb2.ProjectRegisterRequest(project=project.to_flyte_idl(),)
+            _project_pb2.ProjectRegisterRequest(
+                project=project.to_flyte_idl(),
+            )
+        )
+
+    def update_project(self, project):
+        """
+        Update an existing project specified by id.
+        :param flytekit.models.project.Project project:
+        :rtype: flyteidl.admin.project_pb2.ProjectUpdateResponse
+        """
+        super(SynchronousFlyteClient, self).update_project(project.to_flyte_idl())
+
+    def list_projects_paginated(self, limit=100, token=None, filters=None, sort_by=None):
+        """
+        This returns a page of projects.
+
+        .. note ::
+
+            This is a paginated API.  Use the token field in the request to specify a page offset token.
+            The user of the API is responsible for providing this token.
+
+        .. note ::
+
+            If entries are added to the database between requests for different pages, it is possible to receive
+            entries on the second page that also appeared on the first.
+
+        :param int limit: [Optional] The maximum number of entries to return.  Must be greater than 0.  The maximum
+            page size is determined by the Flyte Admin Service configuration.  If limit is greater than the maximum
+            page size, an exception will be raised.
+        :param Text token: [Optional] If specified, this specifies where in the rows of results to skip before reading.
+            If you previously retrieved a page response with token="foo" and you want the next page,
+            specify token="foo". Please see the notes for this function about the caveats of the paginated API.
+        :param list[flytekit.models.filters.Filter] filters: [Optional] If specified, the filters will be applied to
+            the query.  If the filter is not supported, an exception will be raised.
+        :param flytekit.models.admin.common.Sort sort_by: [Optional] If provided, the results will be sorted.
+        :raises grpc.RpcError:
+        :rtype: (list[flytekit.models.Project], Text)
+        """
+        projects = super(SynchronousFlyteClient, self).list_projects(
+            project_list_request=_project_pb2.ProjectListRequest(
+                limit=limit,
+                token=token,
+                filters=_filters.FilterList(filters or []).to_flyte_idl(),
+                sort_by=None if sort_by is None else sort_by.to_flyte_idl(),
+            )
+        )
+        return (
+            [_project.Project.from_flyte_idl(pb) for pb in projects.projects],
+            _six.text_type(projects.token),
         )
 
     ####################################################################################################################
@@ -807,7 +890,9 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         super(SynchronousFlyteClient, self).update_project_domain_attributes(
             _project_domain_attributes_pb2.ProjectDomainAttributesUpdateRequest(
                 attributes=_project_domain_attributes_pb2.ProjectDomainAttributes(
-                    project=project, domain=domain, matching_attributes=matching_attributes.to_flyte_idl(),
+                    project=project,
+                    domain=domain,
+                    matching_attributes=matching_attributes.to_flyte_idl(),
                 )
             )
         )
@@ -842,7 +927,9 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         """
         return super(SynchronousFlyteClient, self).get_project_domain_attributes(
             _project_domain_attributes_pb2.ProjectDomainAttributesGetRequest(
-                project=project, domain=domain, resource_type=resource_type,
+                project=project,
+                domain=domain,
+                resource_type=resource_type,
             )
         )
 
@@ -857,7 +944,10 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         """
         return super(SynchronousFlyteClient, self).get_workflow_attributes(
             _workflow_attributes_pb2.WorkflowAttributesGetRequest(
-                project=project, domain=domain, workflow=workflow, resource_type=resource_type,
+                project=project,
+                domain=domain,
+                workflow=workflow,
+                resource_type=resource_type,
             )
         )
 
@@ -868,5 +958,7 @@ class SynchronousFlyteClient(_RawSynchronousFlyteClient):
         :return:
         """
         return super(SynchronousFlyteClient, self).list_matchable_attributes(
-            _matchable_resource_pb2.ListMatchableAttributesRequest(resource_type=resource_type,)
+            _matchable_resource_pb2.ListMatchableAttributesRequest(
+                resource_type=resource_type,
+            )
         )
