@@ -1,6 +1,6 @@
-from __future__ import absolute_import
-
-import logging as _logging
+import subprocess
+import time
+from typing import List
 
 import six as _six
 from flyteidl.service import admin_pb2_grpc as _admin_service
@@ -15,9 +15,12 @@ from flytekit.clis.auth import credentials as _credentials_access
 from flytekit.clis.sdk_in_container import basic_auth as _basic_auth
 from flytekit.common.exceptions import user as _user_exceptions
 from flytekit.configuration import creds as _creds_config
-from flytekit.configuration.creds import CLIENT_CREDENTIALS_SCOPE as _SCOPE
+from flytekit.configuration.creds import _DEPRECATED_CLIENT_CREDENTIALS_SCOPE as _DEPRECATED_SCOPE
 from flytekit.configuration.creds import CLIENT_ID as _CLIENT_ID
+from flytekit.configuration.creds import COMMAND as _COMMAND
+from flytekit.configuration.creds import OAUTH_SCOPES
 from flytekit.configuration.platform import AUTH as _AUTH
+from flytekit.loggers import cli_logger
 
 
 def _refresh_credentials_standard(flyte_client):
@@ -28,13 +31,32 @@ def _refresh_credentials_standard(flyte_client):
     :return:
     """
 
-    _credentials_access.get_client(flyte_client.url).refresh_access_token()
-    flyte_client.set_access_token(_credentials_access.get_client(flyte_client.url).credentials.access_token)
+    client = _credentials_access.get_client(flyte_client.url)
+    if client.can_refresh_token:
+        client.refresh_access_token()
+
+    flyte_client.set_access_token(client.credentials.access_token)
+
+
+def _get_basic_flow_scopes() -> List[str]:
+    """
+    Merge the scope value between the old scope config option and the new list option.
+
+    :return: The scopes to use for basic auth flow.
+    """
+    deprecated_single_scope = _DEPRECATED_SCOPE.get()
+    if deprecated_single_scope:
+        return [deprecated_single_scope]
+    oauth_scopes = OAUTH_SCOPES.get()
+    if "openid" in oauth_scopes:
+        cli_logger.warning("Basic flow authentication should never use openid.")
+
+    return oauth_scopes
 
 
 def _refresh_credentials_basic(flyte_client):
     """
-    This function is used by the _handle_rpc_error decorator, depending on the AUTH_MODE config object. This handler
+    This function is used by the _handle_rpc_error() decorator, depending on the AUTH_MODE config object. This handler
     is meant for SDK use-cases of auth (like pyflyte, or when users call SDK functions that require access to Admin,
     like when waiting for another workflow to complete from within a task). This function uses basic auth, which means
     the credentials for basic auth must be present from wherever this code is running.
@@ -45,13 +67,36 @@ def _refresh_credentials_basic(flyte_client):
     auth_endpoints = _credentials_access.get_authorization_endpoints(flyte_client.url)
     token_endpoint = auth_endpoints.token_endpoint
     client_secret = _basic_auth.get_secret()
-    _logging.debug("Basic authorization flow with client id {} scope {}".format(_CLIENT_ID.get(), _SCOPE.get()))
+    cli_logger.debug(
+        "Basic authorization flow with client id {} scope {}".format(_CLIENT_ID.get(), _get_basic_flow_scopes())
+    )
     authorization_header = _basic_auth.get_basic_authorization_header(_CLIENT_ID.get(), client_secret)
-    token, expires_in = _basic_auth.get_token(token_endpoint, authorization_header, _SCOPE.get())
-    _logging.info("Retrieved new token, expires in {}".format(expires_in))
+    token, expires_in = _basic_auth.get_token(token_endpoint, authorization_header, _get_basic_flow_scopes())
+    cli_logger.info("Retrieved new token, expires in {}".format(expires_in))
     flyte_client.set_access_token(token)
 
 
+def _refresh_credentials_from_command(flyte_client):
+    """
+    This function is used when the configuration value for AUTH_MODE is set to 'external_process'.
+    It reads an id token generated by an external process started by running the 'command'.
+
+    :param flyte_client: RawSynchronousFlyteClient
+    :return:
+    """
+
+    command = _COMMAND.get()
+    cli_logger.debug("Starting external process to generate id token. Command {}".format(command))
+    try:
+        output = subprocess.run(command, capture_output=True, text=True, check=True)
+    except subprocess.CalledProcessError as e:
+        cli_logger.error("Failed to generate token from command {}".format(command))
+        raise _user_exceptions.FlyteAuthenticationException(
+            "Problems refreshing token with command: " + _six.text_type(e)
+        )
+    flyte_client.set_access_token(output.stdout.strip())
+
+
 def _refresh_credentials_noop(flyte_client):
     pass
 
@@ -61,41 +106,55 @@ def _get_refresh_handler(auth_mode):
         return _refresh_credentials_standard
     elif auth_mode == "basic":
         return _refresh_credentials_basic
+    elif auth_mode == "external_process":
+        return _refresh_credentials_from_command
     else:
         raise ValueError(
             "Invalid auth mode [{}] specified. Please update the creds config to use a valid value".format(auth_mode)
         )
 
 
-def _handle_rpc_error(fn):
-    def handler(*args, **kwargs):
-        """
-        Wraps rpc errors as Flyte exceptions and handles authentication the client.
-        :param args:
-        :param kwargs:
-        :return:
-        """
-        retries = 2
-        try:
-            for i in range(retries):
-                try:
-                    return fn(*args, **kwargs)
-                except _RpcError as e:
-                    if e.code() == _GrpcStatusCode.UNAUTHENTICATED:
-                        if i == (retries - 1):
-                            # Exit the loop and wrap the authentication error.
-                            raise _user_exceptions.FlyteAuthenticationException(_six.text_type(e))
-                        refresh_handler_fn = _get_refresh_handler(_creds_config.AUTH_MODE.get())
-                        refresh_handler_fn(args[0])
-                    else:
-                        raise
-        except _RpcError as e:
-            if e.code() == _GrpcStatusCode.ALREADY_EXISTS:
-                raise _user_exceptions.FlyteEntityAlreadyExistsException(_six.text_type(e))
-            else:
-                raise
+def _handle_rpc_error(retry=False):
+    def decorator(fn):
+        def handler(*args, **kwargs):
+            """
+            Wraps rpc errors as Flyte exceptions and handles authentication the client.
+            :param args:
+            :param kwargs:
+            :return:
+            """
+            max_retries = 3
+            max_wait_time = 1000
+            try:
+                for i in range(max_retries):
+                    try:
+                        return fn(*args, **kwargs)
+                    except _RpcError as e:
+                        if e.code() == _GrpcStatusCode.UNAUTHENTICATED:
+                            # Always retry auth errors.
+                            if i == (max_retries - 1):
+                                # Exit the loop and wrap the authentication error.
+                                raise _user_exceptions.FlyteAuthenticationException(_six.text_type(e))
+                            refresh_handler_fn = _get_refresh_handler(_creds_config.AUTH_MODE.get())
+                            refresh_handler_fn(args[0])
+                        else:
+                            # No more retries if retry=False or max_retries reached.
+                            if (retry is False) or i == (max_retries - 1):
+                                raise
+                            else:
+                                # Retry: Start with 200ms wait-time and exponentially back-off upto 1 second.
+                                wait_time = min(200 * (2 ** i), max_wait_time)
+                                cli_logger.error(f"Non-auth RPC error {e}, sleeping {wait_time}ms and retrying")
+                                time.sleep(wait_time / 1000)
+            except _RpcError as e:
+                if e.code() == _GrpcStatusCode.ALREADY_EXISTS:
+                    raise _user_exceptions.FlyteEntityAlreadyExistsException(_six.text_type(e))
+                else:
+                    raise
 
-    return handler
+        return handler
+
+    return decorator
 
 
 def _handle_invalid_create_request(fn):
@@ -104,8 +163,8 @@ def _handle_invalid_create_request(fn):
             fn(self, create_request)
         except _RpcError as e:
             if e.code() == _GrpcStatusCode.INVALID_ARGUMENT:
-                _logging.error("Error creating Flyte entity because of invalid arguments. Create request: ")
-                _logging.error(_MessageToJson(create_request))
+                cli_logger.error("Error creating Flyte entity because of invalid arguments. Create request: ")
+                cli_logger.error(_MessageToJson(create_request))
 
             # In any case, re-raise since we're not truly handling the error here
             raise e
@@ -140,7 +199,9 @@ class RawSynchronousFlyteClient(object):
             self._channel = _insecure_channel(url, options=list((options or {}).items()))
         else:
             self._channel = _secure_channel(
-                url, credentials or _ssl_channel_credentials(), options=list((options or {}).items()),
+                url,
+                credentials or _ssl_channel_credentials(),
+                options=list((options or {}).items()),
             )
         self._stub = _admin_service.AdminServiceStub(self._channel)
         self._metadata = None
@@ -154,7 +215,12 @@ class RawSynchronousFlyteClient(object):
     def set_access_token(self, access_token):
         # Always set the header to lower-case regardless of what the config is. The grpc libraries that Admin uses
         # to parse the metadata don't change the metadata, but they do automatically lower the key you're looking for.
-        self._metadata = [(_creds_config.AUTHORIZATION_METADATA_KEY.get().lower(), "Bearer {}".format(access_token),)]
+        self._metadata = [
+            (
+                _creds_config.AUTHORIZATION_METADATA_KEY.get().lower(),
+                "Bearer {}".format(access_token),
+            )
+        ]
 
     def force_auth_flow(self):
         refresh_handler_fn = _get_refresh_handler(_creds_config.AUTH_MODE.get())
@@ -166,7 +232,7 @@ class RawSynchronousFlyteClient(object):
     #
     ####################################################################################################################
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     @_handle_invalid_create_request
     def create_task(self, task_create_request):
         """
@@ -188,7 +254,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.CreateTask(task_create_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_task_ids_paginated(self, identifier_list_request):
         """
         This returns a page of identifiers for the tasks for a given project and domain. Filters can also be
@@ -214,7 +280,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.ListTaskIds(identifier_list_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_tasks_paginated(self, resource_list_request):
         """
         This returns a page of task metadata for tasks in a given project and domain.  Optionally,
@@ -236,7 +302,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.ListTasks(resource_list_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_task(self, get_object_request):
         """
         This returns a single task for a given identifier.
@@ -253,7 +319,7 @@ class RawSynchronousFlyteClient(object):
     #
     ####################################################################################################################
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     @_handle_invalid_create_request
     def create_workflow(self, workflow_create_request):
         """
@@ -275,7 +341,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.CreateWorkflow(workflow_create_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_workflow_ids_paginated(self, identifier_list_request):
         """
         This returns a page of identifiers for the workflows for a given project and domain. Filters can also be
@@ -301,7 +367,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.ListWorkflowIds(identifier_list_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_workflows_paginated(self, resource_list_request):
         """
         This returns a page of workflow meta-information for workflows in a given project and domain.  Optionally,
@@ -323,7 +389,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.ListWorkflows(resource_list_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_workflow(self, get_object_request):
         """
         This returns a single workflow for a given identifier.
@@ -340,7 +406,7 @@ class RawSynchronousFlyteClient(object):
     #
     ####################################################################################################################
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     @_handle_invalid_create_request
     def create_launch_plan(self, launch_plan_create_request):
         """
@@ -365,7 +431,7 @@ class RawSynchronousFlyteClient(object):
 
     # TODO: List endpoints when they come in
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_launch_plan(self, object_get_request):
         """
         Retrieves a launch plan entity.
@@ -375,7 +441,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetLaunchPlan(object_get_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_active_launch_plan(self, active_launch_plan_request):
         """
         Retrieves a launch plan entity.
@@ -385,7 +451,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetActiveLaunchPlan(active_launch_plan_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     def update_launch_plan(self, update_request):
         """
         Allows updates to a launch plan at a given identifier.  Currently, a launch plan may only have it's state
@@ -396,7 +462,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.UpdateLaunchPlan(update_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_launch_plan_ids_paginated(self, identifier_list_request):
         """
         Lists launch plan named identifiers for a given project and domain.
@@ -406,7 +472,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.ListLaunchPlanIds(identifier_list_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_launch_plans_paginated(self, resource_list_request):
         """
         Lists Launch Plans for a given Identifer (project, domain, name)
@@ -416,7 +482,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.ListLaunchPlans(resource_list_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_active_launch_plans_paginated(self, active_launch_plan_list_request):
         """
         Lists Active Launch Plans for a given (project, domain)
@@ -432,7 +498,7 @@ class RawSynchronousFlyteClient(object):
     #
     ####################################################################################################################
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     def update_named_entity(self, update_named_entity_request):
         """
         :param flyteidl.admin.common_pb2.NamedEntityUpdateRequest update_named_entity_request:
@@ -446,7 +512,7 @@ class RawSynchronousFlyteClient(object):
     #
     ####################################################################################################################
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     def create_execution(self, create_execution_request):
         """
         This will create an execution for the given execution spec.
@@ -455,7 +521,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.CreateExecution(create_execution_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_execution(self, get_object_request):
         """
         Returns an execution of a workflow entity.
@@ -465,7 +531,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetExecution(get_object_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_execution_data(self, get_execution_data_request):
         """
         Returns signed URLs to LiteralMap blobs for an execution's inputs and outputs (when available).
@@ -475,7 +541,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetExecutionData(get_execution_data_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_executions_paginated(self, resource_list_request):
         """
         Lists the executions for a given identifier.
@@ -485,7 +551,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.ListExecutions(resource_list_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     def terminate_execution(self, terminate_execution_request):
         """
         :param flyteidl.admin.execution_pb2.TerminateExecutionRequest terminate_execution_request:
@@ -493,7 +559,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.TerminateExecution(terminate_execution_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     def relaunch_execution(self, relaunch_execution_request):
         """
         :param flyteidl.admin.execution_pb2.ExecutionRelaunchRequest relaunch_execution_request:
@@ -507,7 +573,7 @@ class RawSynchronousFlyteClient(object):
     #
     ####################################################################################################################
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_node_execution(self, node_execution_request):
         """
         :param flyteidl.admin.node_execution_pb2.NodeExecutionGetRequest node_execution_request:
@@ -515,7 +581,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetNodeExecution(node_execution_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_node_execution_data(self, get_node_execution_data_request):
         """
         Returns signed URLs to LiteralMap blobs for a node execution's inputs and outputs (when available).
@@ -525,7 +591,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetNodeExecutionData(get_node_execution_data_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_node_executions_paginated(self, node_execution_list_request):
         """
         :param flyteidl.admin.node_execution_pb2.NodeExecutionListRequest node_execution_list_request:
@@ -533,7 +599,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.ListNodeExecutions(node_execution_list_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_node_executions_for_task_paginated(self, node_execution_for_task_list_request):
         """
         :param flyteidl.admin.node_execution_pb2.NodeExecutionListRequest node_execution_for_task_list_request:
@@ -547,7 +613,7 @@ class RawSynchronousFlyteClient(object):
     #
     ####################################################################################################################
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_task_execution(self, task_execution_request):
         """
         :param flyteidl.admin.task_execution_pb2.TaskExecutionGetRequest task_execution_request:
@@ -555,7 +621,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetTaskExecution(task_execution_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_task_execution_data(self, get_task_execution_data_request):
         """
         Returns signed URLs to LiteralMap blobs for a task execution's inputs and outputs (when available).
@@ -565,7 +631,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetTaskExecutionData(get_task_execution_data_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_task_executions_paginated(self, task_execution_list_request):
         """
         :param flyteidl.admin.task_execution_pb2.TaskExecutionListRequest task_execution_list_request:
@@ -579,7 +645,7 @@ class RawSynchronousFlyteClient(object):
     #
     ####################################################################################################################
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_projects(self, project_list_request):
         """
         This will return a list of the projects registered with the Flyte Admin Service
@@ -588,7 +654,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.ListProjects(project_list_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     def register_project(self, project_register_request):
         """
         Registers a project along with a set of domains.
@@ -597,12 +663,21 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.RegisterProject(project_register_request, metadata=self._metadata)
 
+    @_handle_rpc_error()
+    def update_project(self, project):
+        """
+        Update an existing project specified by id.
+        :param flyteidl.admin.project_pb2.Project project:
+        :rtype: flyteidl.admin.project_pb2.ProjectUpdateResponse
+        """
+        return self._stub.UpdateProject(project)
+
     ####################################################################################################################
     #
     #  Matching Attributes Endpoints
     #
     ####################################################################################################################
-    @_handle_rpc_error
+    @_handle_rpc_error()
     def update_project_domain_attributes(self, project_domain_attributes_update_request):
         """
         This updates the attributes for a project and domain registered with the Flyte Admin Service
@@ -613,7 +688,7 @@ class RawSynchronousFlyteClient(object):
             project_domain_attributes_update_request, metadata=self._metadata
         )
 
-    @_handle_rpc_error
+    @_handle_rpc_error()
     def update_workflow_attributes(self, workflow_attributes_update_request):
         """
         This updates the attributes for a project, domain, and workflow registered with the Flyte Admin Service
@@ -622,7 +697,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.UpdateWorkflowAttributes(workflow_attributes_update_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_project_domain_attributes(self, project_domain_attributes_get_request):
         """
         This fetches the attributes for a project and domain registered with the Flyte Admin Service
@@ -631,7 +706,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetProjectDomainAttributes(project_domain_attributes_get_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def get_workflow_attributes(self, workflow_attributes_get_request):
         """
         This fetches the attributes for a project, domain, and workflow registered with the Flyte Admin Service
@@ -640,7 +715,7 @@ class RawSynchronousFlyteClient(object):
         """
         return self._stub.GetWorkflowAttributes(workflow_attributes_get_request, metadata=self._metadata)
 
-    @_handle_rpc_error
+    @_handle_rpc_error(retry=True)
     def list_matchable_attributes(self, matchable_attributes_list_request):
         """
         This fetches the attributes for a specific resource type registered with the Flyte Admin Service
