@@ -1,17 +1,13 @@
-from __future__ import absolute_import, print_function
-
 import logging as _logging
 import os as _os
+from pathlib import Path
 
 import click
 
-try:
-    from pathlib import Path
-except ImportError:
-    from pathlib2 import Path  # python 2 backport
-
-from flytekit.clis.sdk_in_container.constants import CTX_DOMAIN, CTX_PACKAGES, CTX_PROJECT, CTX_VERSION
+from flytekit.clis.sdk_in_container.constants import CTX_PACKAGES
+from flytekit.clis.sdk_in_container.fast_register import fast_register
 from flytekit.clis.sdk_in_container.launch_plan import launch_plans
+from flytekit.clis.sdk_in_container.package import package
 from flytekit.clis.sdk_in_container.register import register
 from flytekit.clis.sdk_in_container.serialize import serialize
 from flytekit.configuration import internal as _internal_config
@@ -19,42 +15,45 @@ from flytekit.configuration import platform as _platform_config
 from flytekit.configuration import sdk as _sdk_config
 from flytekit.configuration import set_flyte_config_file
 from flytekit.configuration.internal import CONFIGURATION_PATH
-from flytekit.configuration.internal import IMAGE as _IMAGE
-from flytekit.configuration.internal import look_up_version_from_image_tag as _look_up_version_from_image_tag
 from flytekit.configuration.platform import URL as _URL
 from flytekit.configuration.sdk import WORKFLOW_PACKAGES as _WORKFLOW_PACKAGES
 
 
+def validate_package(ctx, param, values):
+    for val in values:
+        if "/" in val or "-" in val or "\\" in val:
+            raise click.BadParameter(
+                f"Illegal package value {val} for parameter: {param}. Expected for the form [a.b.c]"
+            )
+    return values
+
+
 @click.group("pyflyte", invoke_without_command=True)
 @click.option(
-    "-p",
-    "--project",
-    required=True,
+    "-c",
+    "--config",
+    required=False,
     type=str,
-    help="Flyte project to use. You can have more than one project per repo",
-)
-@click.option(
-    "-d", "--domain", required=True, type=str, help="This is usually development, staging, or production",
-)
-@click.option(
-    "-c", "--config", required=False, type=str, help="Path to config file for use within container",
+    help="Path to config file for use within container",
 )
 @click.option(
     "-k",
     "--pkgs",
     required=False,
     multiple=True,
+    callback=validate_package,
     help="Dot separated python packages to operate on.  Multiple may be specified  Please note that this "
     "option will override the option specified in the configuration file, or environment variable",
 )
 @click.option(
-    "-v", "--version", required=False, type=str, help="This is the version to apply globally for this " "context",
-)
-@click.option(
-    "-i", "--insecure", required=False, type=bool, help="Do not use SSL to connect to Admin",
+    "-i",
+    "--insecure",
+    required=False,
+    type=bool,
+    help="Disable SSL when connecting to Flyte backend.",
 )
 @click.pass_context
-def main(ctx, project, domain, config=None, pkgs=None, version=None, insecure=None):
+def main(ctx, config=None, pkgs=None, insecure=None):
     """
     Entrypoint for all the user commands.
     """
@@ -66,10 +65,6 @@ def main(ctx, project, domain, config=None, pkgs=None, version=None, insecure=No
         _logging.getLogger().setLevel(log_level)
 
     ctx.obj = dict()
-    ctx.obj[CTX_PROJECT] = project
-    ctx.obj[CTX_DOMAIN] = domain
-    version = version or _look_up_version_from_image_tag(_IMAGE.get())
-    ctx.obj[CTX_VERSION] = version
 
     # Determine SSL.  Note that the insecure option in this command is not a flag because we want to look
     # up configuration settings if it's missing.  If the command line option says insecure but the config object
@@ -84,10 +79,6 @@ def main(ctx, project, domain, config=None, pkgs=None, version=None, insecure=No
         pkgs = _WORKFLOW_PACKAGES.get()
     ctx.obj[CTX_PACKAGES] = pkgs
 
-    _os.environ[_internal_config.PROJECT.env_var] = project
-    _os.environ[_internal_config.DOMAIN.env_var] = domain
-    _os.environ[_internal_config.VERSION.env_var] = version
-
 
 def update_configuration_file(config_file_path):
     """
@@ -99,7 +90,8 @@ def update_configuration_file(config_file_path):
     configuration_file = Path(config_file_path or CONFIGURATION_PATH.get())
     if configuration_file.is_file():
         click.secho(
-            "Using configuration file at {}".format(configuration_file.absolute().as_posix()), fg="green",
+            "Using configuration file at {}".format(configuration_file.absolute().as_posix()),
+            fg="green",
         )
         set_flyte_config_file(configuration_file.as_posix())
     else:
@@ -114,8 +106,10 @@ def update_configuration_file(config_file_path):
 
 
 main.add_command(register)
+main.add_command(fast_register)
 main.add_command(serialize)
 main.add_command(launch_plans)
+main.add_command(package)
 
 if __name__ == "__main__":
     main()
