@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import six as _six
 
 from flytekit.common import sdk_bases as _sdk_bases
@@ -7,7 +5,7 @@ from flytekit.common.exceptions import user as _user_exceptions
 from flytekit.models.core import identifier as _core_identifier
 
 
-class Identifier(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _core_identifier.Identifier)):
+class Identifier(_core_identifier.Identifier, metaclass=_sdk_bases.ExtendedSdkType):
 
     _STRING_TO_TYPE_MAP = {
         "lp": _core_identifier.ResourceType.LAUNCH_PLAN,
@@ -23,7 +21,11 @@ class Identifier(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _core_identifie
         :rtype: Identifier
         """
         return cls(
-            base_model.resource_type, base_model.project, base_model.domain, base_model.name, base_model.version,
+            base_model.resource_type,
+            base_model.project,
+            base_model.domain,
+            base_model.name,
+            base_model.version,
         )
 
     @classmethod
@@ -61,16 +63,18 @@ class Identifier(_six.with_metaclass(_sdk_bases.ExtendedSdkType, _core_identifie
         )
 
 
-class WorkflowExecutionIdentifier(
-    _six.with_metaclass(_sdk_bases.ExtendedSdkType, _core_identifier.WorkflowExecutionIdentifier)
-):
+class WorkflowExecutionIdentifier(_core_identifier.WorkflowExecutionIdentifier, metaclass=_sdk_bases.ExtendedSdkType):
     @classmethod
     def promote_from_model(cls, base_model):
         """
         :param flytekit.models.core.identifier.WorkflowExecutionIdentifier base_model:
         :rtype: WorkflowExecutionIdentifier
         """
-        return cls(base_model.project, base_model.domain, base_model.name,)
+        return cls(
+            base_model.project,
+            base_model.domain,
+            base_model.name,
+        )
 
     @classmethod
     def from_python_std(cls, string):
@@ -95,15 +99,17 @@ class WorkflowExecutionIdentifier(
                 "The provided string could not be parsed. The first element of an execution identifier must be 'ex'.",
             )
 
-        return cls(project, domain, name,)
+        return cls(
+            project,
+            domain,
+            name,
+        )
 
     def __str__(self):
         return "ex:{}:{}:{}".format(self.project, self.domain, self.name)
 
 
-class TaskExecutionIdentifier(
-    _six.with_metaclass(_sdk_bases.ExtendedSdkType, _core_identifier.TaskExecutionIdentifier)
-):
+class TaskExecutionIdentifier(_core_identifier.TaskExecutionIdentifier, metaclass=_sdk_bases.ExtendedSdkType):
     @classmethod
     def promote_from_model(cls, base_model):
         """
@@ -142,7 +148,8 @@ class TaskExecutionIdentifier(
         return cls(
             task_id=Identifier(_core_identifier.ResourceType.TASK, tp, td, tn, tv),
             node_execution_id=_core_identifier.NodeExecutionIdentifier(
-                node_id=node_id, execution_id=_core_identifier.WorkflowExecutionIdentifier(ep, ed, en),
+                node_id=node_id,
+                execution_id=_core_identifier.WorkflowExecutionIdentifier(ep, ed, en),
             ),
             retry_attempt=int(retry),
         )
