@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 from flytekit.common.exceptions.base import FlyteException as _FlyteException
 from flytekit.common.exceptions.base import FlyteRecoverableException as _Recoverable
 
@@ -36,7 +34,10 @@ class FlyteTypeException(FlyteUserException, TypeError):
     def __init__(self, received_type, expected_type, additional_msg=None, received_value=None):
         super(FlyteTypeException, self).__init__(
             self._create_verbose_message(
-                received_type, expected_type, received_value=received_value, additional_msg=additional_msg,
+                received_type,
+                expected_type,
+                received_value=received_value,
+                additional_msg=additional_msg,
             )
         )
 
