@@ -1,7 +1,3 @@
-from __future__ import absolute_import
-
-import six as _six
-
 from flytekit.common.exceptions import user as _user_exceptions
 from flytekit.common.types import base_sdk_types as _base_sdk_types
 from flytekit.common.types.impl import blobs as _blob_impl
@@ -45,7 +41,7 @@ class BlobInstantiator(_base_sdk_types.InstantiableType):
 
 
 # TODO: Make blobs and schemas pluggable
-class Blob(_six.with_metaclass(BlobInstantiator, _base_sdk_types.FlyteSdkValue)):
+class Blob(_base_sdk_types.FlyteSdkValue, metaclass=BlobInstantiator):
     @classmethod
     def from_string(cls, string_value):
         """
@@ -163,7 +159,7 @@ class MultiPartBlobInstantiator(_base_sdk_types.InstantiableType):
             return super(MultiPartBlobInstantiator, cls).__call__(*args, **kwargs)
 
 
-class MultiPartBlob(_six.with_metaclass(MultiPartBlobInstantiator, _base_sdk_types.FlyteSdkValue)):
+class MultiPartBlob(_base_sdk_types.FlyteSdkValue, metaclass=MultiPartBlobInstantiator):
     @classmethod
     def from_string(cls, string_value):
         """
@@ -172,7 +168,8 @@ class MultiPartBlob(_six.with_metaclass(MultiPartBlobInstantiator, _base_sdk_typ
         """
         if not string_value:
             _user_exceptions.FlyteValueException(
-                string_value, "Cannot create a MultiPartBlob from the provided path " "value.",
+                string_value,
+                "Cannot create a MultiPartBlob from the provided path " "value.",
             )
         return cls(_blob_impl.MultiPartBlob.from_string(string_value, mode="rb"))
 
@@ -205,7 +202,10 @@ class MultiPartBlob(_six.with_metaclass(MultiPartBlobInstantiator, _base_sdk_typ
         :rtype: flytekit.models.types.LiteralType
         """
         return _idl_types.LiteralType(
-            blob=_core_types.BlobType(format="", dimensionality=_core_types.BlobType.BlobDimensionality.MULTIPART,)
+            blob=_core_types.BlobType(
+                format="",
+                dimensionality=_core_types.BlobType.BlobDimensionality.MULTIPART,
+            )
         )
 
     @classmethod
@@ -283,7 +283,7 @@ class CsvInstantiator(BlobInstantiator):
             return super(CsvInstantiator, cls).__call__(*args, **kwargs)
 
 
-class CSV(_six.with_metaclass(CsvInstantiator, Blob)):
+class CSV(Blob, metaclass=CsvInstantiator):
     @classmethod
     def from_string(cls, string_value):
         """
@@ -325,7 +325,10 @@ class CSV(_six.with_metaclass(CsvInstantiator, Blob)):
         :rtype: flytekit.models.types.LiteralType
         """
         return _idl_types.LiteralType(
-            blob=_core_types.BlobType(format="csv", dimensionality=_core_types.BlobType.BlobDimensionality.SINGLE,)
+            blob=_core_types.BlobType(
+                format="csv",
+                dimensionality=_core_types.BlobType.BlobDimensionality.SINGLE,
+            )
         )
 
     @classmethod
@@ -386,7 +389,7 @@ class MultiPartCsvInstantiator(MultiPartBlobInstantiator):
             return super(MultiPartCsvInstantiator, cls).__call__(*args, **kwargs)
 
 
-class MultiPartCSV(_six.with_metaclass(MultiPartCsvInstantiator, MultiPartBlob)):
+class MultiPartCSV(MultiPartBlob, metaclass=MultiPartCsvInstantiator):
     @classmethod
     def from_string(cls, string_value):
         """
@@ -395,7 +398,8 @@ class MultiPartCSV(_six.with_metaclass(MultiPartCsvInstantiator, MultiPartBlob))
         """
         if not string_value:
             _user_exceptions.FlyteValueException(
-                string_value, "Cannot create a MultiPartCSV from the provided path value.",
+                string_value,
+                "Cannot create a MultiPartCSV from the provided path value.",
             )
         return cls(_blob_impl.MultiPartBlob.from_string(string_value, format="csv", mode="r"))
 
@@ -432,7 +436,10 @@ class MultiPartCSV(_six.with_metaclass(MultiPartCsvInstantiator, MultiPartBlob))
         :rtype: flytekit.models.types.LiteralType
         """
         return _idl_types.LiteralType(
-            blob=_core_types.BlobType(format="csv", dimensionality=_core_types.BlobType.BlobDimensionality.MULTIPART,)
+            blob=_core_types.BlobType(
+                format="csv",
+                dimensionality=_core_types.BlobType.BlobDimensionality.MULTIPART,
+            )
         )
 
     @classmethod
