@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 from flytekit.configuration import common as _config_common
 
 WORKFLOW_PACKAGES = _config_common.FlyteStringListConfigurationEntry("sdk", "workflow_packages", default=[])
@@ -33,7 +31,7 @@ SDK_PYTHON_VENV = _config_common.FlyteStringListConfigurationEntry("sdk", "pytho
 This is a list of commands/args which will be prefixed to the entrypoint command by SDK.
 """
 
-ROLE = _config_common.FlyteRequiredStringConfigurationEntry("sdk", "role")
+ROLE = _config_common.FlyteStringConfigurationEntry("sdk", "role")
 """
 This is the role the SDK will use by default to execute workflows.  For example, in AWS this should be an IAM role
 string.
@@ -80,3 +78,10 @@ PARQUET_ENGINE = _config_common.FlyteStringConfigurationEntry("sdk", "parquet_en
 """
 This is the parquet engine to use when reading data from parquet files.
 """
+
+FAST_REGISTRATION_DIR = _config_common.FlyteStringConfigurationEntry("sdk", "fast_registration_dir")
+"""
+This is the remote directory where fast-registered code will be uploaded to.
+Users calling fast-execute need write permission to this directory.
+Furthermore, it is important that whichever role executes your workflow has read access to this directory.
+"""
