@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import logging as _logging
 import os as _os
 import traceback as _traceback
@@ -9,7 +7,7 @@ import six as _six
 from deprecated import deprecated as _deprecated
 from flyteidl.core import literals_pb2 as _literals_pb2
 
-from flytekit import __version__ as _api_version
+import flytekit
 from flytekit.clients.friendly import SynchronousFlyteClient as _SynchronousFlyteClient
 from flytekit.clients.helpers import iterate_node_executions as _iterate_node_executions
 from flytekit.clients.helpers import iterate_task_executions as _iterate_task_executions
@@ -52,6 +50,12 @@ class _FlyteClientManager(object):
         return type(self)._CLIENT
 
 
+# This is a simple helper function that ties the client together with the configuration construct.
+# This will be refactored away when we move to a heavier context object.
+def get_client() -> _SynchronousFlyteClient:
+    return _FlyteClientManager(_platform_config.URL.get(), insecure=_platform_config.INSECURE.get()).client
+
+
 class FlyteEngineFactory(_common_engine.BaseExecutionEngineFactory):
     def get_workflow(self, sdk_workflow):
         """
@@ -95,6 +99,10 @@ class FlyteEngineFactory(_common_engine.BaseExecutionEngineFactory):
         """
         return FlyteWorkflowExecution(wf_exec)
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def fetch_workflow_execution(self, wf_exec_id):
         """
         :param flytekit.models.core.identifier.WorkflowExecutionIdentifier wf_exec_id:
@@ -104,6 +112,10 @@ class FlyteEngineFactory(_common_engine.BaseExecutionEngineFactory):
             _platform_config.URL.get(), insecure=_platform_config.INSECURE.get()
         ).client.get_execution(wf_exec_id)
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def fetch_task(self, task_id):
         """
         Queries Admin for an existing Admin task
@@ -114,6 +126,10 @@ class FlyteEngineFactory(_common_engine.BaseExecutionEngineFactory):
             _platform_config.URL.get(), insecure=_platform_config.INSECURE.get()
         ).client.get_task(task_id)
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def fetch_latest_task(self, named_task):
         """
         Fetches the latest task
@@ -123,10 +139,16 @@ class FlyteEngineFactory(_common_engine.BaseExecutionEngineFactory):
         task_list, _ = _FlyteClientManager(
             _platform_config.URL.get(), insecure=_platform_config.INSECURE.get()
         ).client.list_tasks_paginated(
-            named_task, limit=1, sort_by=_common.Sort("created_at", _common.Sort.Direction.DESCENDING),
+            named_task,
+            limit=1,
+            sort_by=_common.Sort("created_at", _common.Sort.Direction.DESCENDING),
         )
         return task_list[0] if task_list else None
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def fetch_launch_plan(self, launch_plan_id):
         """
         :param flytekit.models.core.identifier.Identifier launch_plan_id: This identifier should have a resource
@@ -145,6 +167,10 @@ class FlyteEngineFactory(_common_engine.BaseExecutionEngineFactory):
                 _platform_config.URL.get(), insecure=_platform_config.INSECURE.get()
             ).client.get_active_launch_plan(named_entity_id)
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def fetch_workflow(self, workflow_id):
         """
         :param flytekit.models.core.identifier.Identifier workflow_id: This identifier should have a resource
@@ -157,6 +183,10 @@ class FlyteEngineFactory(_common_engine.BaseExecutionEngineFactory):
 
 
 class FlyteLaunchPlan(_common_engine.BaseLaunchPlanLauncher):
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def register(self, identifier):
         client = _FlyteClientManager(_platform_config.URL.get(), insecure=_platform_config.INSECURE.get()).client
         try:
@@ -179,9 +209,19 @@ class FlyteLaunchPlan(_common_engine.BaseLaunchPlanLauncher):
         Deprecated. Use launch instead.
         """
         return self.launch(
-            project, domain, name, inputs, notification_overrides, label_overrides, annotation_overrides,
+            project,
+            domain,
+            name,
+            inputs,
+            notification_overrides,
+            label_overrides,
+            annotation_overrides,
         )
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def launch(
         self,
         project,
@@ -235,6 +275,10 @@ class FlyteLaunchPlan(_common_engine.BaseLaunchPlanLauncher):
             exec_id = _identifier.WorkflowExecutionIdentifier(project, domain, name)
         return client.get_execution(exec_id)
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def update(self, identifier, state):
         """
         :param flytekit.models.core.identifier.Identifier identifier: Identifier for launch plan to update
@@ -246,16 +290,30 @@ class FlyteLaunchPlan(_common_engine.BaseLaunchPlanLauncher):
 
 
 class FlyteWorkflow(_common_engine.BaseWorkflowExecutor):
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def register(self, identifier):
         client = _FlyteClientManager(_platform_config.URL.get(), insecure=_platform_config.INSECURE.get()).client
         try:
             sub_workflows = self.sdk_workflow.get_sub_workflows()
-            return client.create_workflow(identifier, _workflow_model.WorkflowSpec(self.sdk_workflow, sub_workflows,),)
+            return client.create_workflow(
+                identifier,
+                _workflow_model.WorkflowSpec(
+                    self.sdk_workflow,
+                    sub_workflows,
+                ),
+            )
         except _user_exceptions.FlyteEntityAlreadyExistsException:
             pass
 
 
 class FlyteTask(_common_engine.BaseTaskExecutor):
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def register(self, identifier):
         client = _FlyteClientManager(_platform_config.URL.get(), insecure=_platform_config.INSECURE.get()).client
         try:
@@ -306,7 +364,7 @@ class FlyteTask(_common_engine.BaseTaskExecutor):
                                             "exec_domain": _internal_config.EXECUTION_DOMAIN.get(),
                                             "exec_workflow": _internal_config.EXECUTION_WORKFLOW.get(),
                                             "exec_launchplan": _internal_config.EXECUTION_LAUNCHPLAN.get(),
-                                            "api_version": _api_version,
+                                            "api_version": flytekit.__version__,
                                         },
                                     ),
                                     logging=_logging,
@@ -329,7 +387,9 @@ class FlyteTask(_common_engine.BaseTaskExecutor):
                             exc_str = _traceback.format_exc()
                             output_file_dict[_constants.ERROR_FILE_NAME] = _error_models.ErrorDocument(
                                 _error_models.ContainerError(
-                                    "SYSTEM:Unknown", exc_str, _error_models.ContainerError.Kind.RECOVERABLE,
+                                    "SYSTEM:Unknown",
+                                    exc_str,
+                                    _error_models.ContainerError.Kind.RECOVERABLE,
                                 )
                             )
                             _logging.error(exc_str)
@@ -338,9 +398,15 @@ class FlyteTask(_common_engine.BaseTaskExecutor):
                             for k, v in _six.iteritems(output_file_dict):
                                 _common_utils.write_proto_to_file(v.to_flyte_idl(), _os.path.join(temp_dir.name, k))
                             _data_proxy.Data.put_data(
-                                temp_dir.name, context["output_prefix"], is_multipart=True,
+                                temp_dir.name,
+                                context["output_prefix"],
+                                is_multipart=True,
                             )
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def launch(
         self,
         project,
@@ -383,7 +449,8 @@ class FlyteTask(_common_engine.BaseTaskExecutor):
                 )
                 assumable_iam_role = _sdk_config.ROLE.get()
             auth_role = _common_models.AuthRole(
-                assumable_iam_role=assumable_iam_role, kubernetes_service_account=kubernetes_service_account,
+                assumable_iam_role=assumable_iam_role,
+                kubernetes_service_account=kubernetes_service_account,
             )
 
         try:
@@ -414,6 +481,10 @@ class FlyteTask(_common_engine.BaseTaskExecutor):
 
 
 class FlyteWorkflowExecution(_common_engine.BaseWorkflowExecution):
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_node_executions(self, filters=None):
         """
         :param list[flytekit.models.filters.Filter] filters:
@@ -424,6 +495,10 @@ class FlyteWorkflowExecution(_common_engine.BaseWorkflowExecution):
             v.id.node_id: v for v in _iterate_node_executions(client, self.sdk_workflow_execution.id, filters=filters)
         }
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def sync(self):
         """
         :rtype: None
@@ -431,6 +506,10 @@ class FlyteWorkflowExecution(_common_engine.BaseWorkflowExecution):
         client = _FlyteClientManager(_platform_config.URL.get(), insecure=_platform_config.INSECURE.get()).client
         self.sdk_workflow_execution._closure = client.get_execution(self.sdk_workflow_execution.id).closure
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_inputs(self):
         """
         :rtype: flytekit.models.literals.LiteralMap
@@ -451,6 +530,10 @@ class FlyteWorkflowExecution(_common_engine.BaseWorkflowExecution):
                 )
         return _literals.LiteralMap({})
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_outputs(self):
         """
         :rtype: flytekit.models.literals.LiteralMap
@@ -471,6 +554,10 @@ class FlyteWorkflowExecution(_common_engine.BaseWorkflowExecution):
                 )
         return _literals.LiteralMap({})
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def terminate(self, cause):
         """
         :param Text cause:
@@ -481,6 +568,10 @@ class FlyteWorkflowExecution(_common_engine.BaseWorkflowExecution):
 
 
 class FlyteNodeExecution(_common_engine.BaseNodeExecution):
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_task_executions(self):
         """
         :rtype: list[flytekit.common.tasks.executions.SdkTaskExecution]
@@ -488,12 +579,20 @@ class FlyteNodeExecution(_common_engine.BaseNodeExecution):
         client = _FlyteClientManager(_platform_config.URL.get(), insecure=_platform_config.INSECURE.get()).client
         return list(_iterate_task_executions(client, self.sdk_node_execution.id))
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_subworkflow_executions(self):
         """
         :rtype: list[flytekit.common.workflow_execution.SdkWorkflowExecution]
         """
         raise NotImplementedError("Cannot retrieve sub-workflow information from a node execution yet.")
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_inputs(self):
         """
         :rtype: flytekit.models.literals.LiteralMap
@@ -514,6 +613,10 @@ class FlyteNodeExecution(_common_engine.BaseNodeExecution):
                 )
         return _literals.LiteralMap({})
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_outputs(self):
         """
         :rtype: flytekit.models.literals.LiteralMap
@@ -534,6 +637,10 @@ class FlyteNodeExecution(_common_engine.BaseNodeExecution):
                 )
         return _literals.LiteralMap({})
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def sync(self):
         """
         :rtype: None
@@ -543,6 +650,10 @@ class FlyteNodeExecution(_common_engine.BaseNodeExecution):
 
 
 class FlyteTaskExecution(_common_engine.BaseTaskExecution):
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_inputs(self):
         """
         :rtype: flytekit.models.literals.LiteralMap
@@ -563,6 +674,10 @@ class FlyteTaskExecution(_common_engine.BaseTaskExecution):
                 )
         return _literals.LiteralMap({})
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_outputs(self):
         """
         :rtype: flytekit.models.literals.LiteralMap
@@ -583,6 +698,10 @@ class FlyteTaskExecution(_common_engine.BaseTaskExecution):
                 )
         return _literals.LiteralMap({})
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def sync(self):
         """
         :rtype: None
@@ -590,6 +709,10 @@ class FlyteTaskExecution(_common_engine.BaseTaskExecution):
         client = _FlyteClientManager(_platform_config.URL.get(), insecure=_platform_config.INSECURE.get()).client
         self.sdk_task_execution._closure = client.get_task_execution(self.sdk_task_execution.id).closure
 
+    @_deprecated(
+        reason="Objects should access client directly, will be removed by 1.0",
+        version="0.13.0",
+    )
     def get_child_executions(self, filters=None):
         """
         :param list[flytekit.models.filters.Filter] filters:
@@ -599,6 +722,8 @@ class FlyteTaskExecution(_common_engine.BaseTaskExecution):
         return {
             v.id.node_id: v
             for v in _iterate_node_executions(
-                client, task_execution_identifier=self.sdk_task_execution.id, filters=filters,
+                client,
+                task_execution_identifier=self.sdk_task_execution.id,
+                filters=filters,
             )
         }
