@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import abc as _abc
 import json as _json
 
@@ -40,7 +38,7 @@ class FlyteType(FlyteABCMeta):
         pass
 
 
-class FlyteIdlEntity(_six.with_metaclass(FlyteType, object)):
+class FlyteIdlEntity(object, metaclass=FlyteType):
     def __eq__(self, other):
         return isinstance(other, FlyteIdlEntity) and other.to_flyte_idl() == self.to_flyte_idl()
 
@@ -244,7 +242,13 @@ class PagerDutyNotification(FlyteIdlEntity):
 
 
 class Notification(FlyteIdlEntity):
-    def __init__(self, phases, email=None, pager_duty=None, slack=None):
+    def __init__(
+        self,
+        phases,
+        email: EmailNotification = None,
+        pager_duty: PagerDutyNotification = None,
+        slack: SlackNotification = None,
+    ):
         """
         Represents a structure for notifications based on execution status.
 
@@ -414,8 +418,6 @@ class AuthRole(FlyteIdlEntity):
         :param Text kubernetes_service_account: Provides an identity for workflow execution resources. Flyte deployment
             administrators are responsible for handling permissions as they relate to the service account.
         """
-        if assumable_iam_role and kubernetes_service_account:
-            raise ValueError("Only one of assumable_iam_role or kubernetes_service_account can be set")
         self._assumable_iam_role = assumable_iam_role
         self._kubernetes_service_account = kubernetes_service_account
 
@@ -451,10 +453,8 @@ class AuthRole(FlyteIdlEntity):
         :rtype: Auth
         """
         return cls(
-            assumable_iam_role=pb2_object.assumable_iam_role if pb2_object.HasField("assumable_iam_role") else None,
-            kubernetes_service_account=pb2_object.kubernetes_service_account
-            if pb2_object.HasField("kubernetes_service_account")
-            else None,
+            assumable_iam_role=pb2_object.assumable_iam_role,
+            kubernetes_service_account=pb2_object.kubernetes_service_account,
         )
 
 
