@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 from flyteidl.core import execution_pb2 as _execution_pb2
 
 from flytekit.models import common as _common
@@ -149,7 +147,11 @@ class ExecutionError(_common.FlyteIdlEntity):
         """
         :rtype: flyteidl.core.execution_pb2.ExecutionError
         """
-        return _execution_pb2.ExecutionError(code=self.code, message=self.message, error_uri=self.error_uri,)
+        return _execution_pb2.ExecutionError(
+            code=self.code,
+            message=self.message,
+            error_uri=self.error_uri,
+        )
 
     @classmethod
     def from_flyte_idl(cls, p):
@@ -157,7 +159,11 @@ class ExecutionError(_common.FlyteIdlEntity):
         :param flyteidl.core.execution_pb2.ExecutionError p:
         :rtype: ExecutionError
         """
-        return cls(code=p.code, message=p.message, error_uri=p.error_uri,)
+        return cls(
+            code=p.code,
+            message=p.message,
+            error_uri=p.error_uri,
+        )
 
 
 class TaskLog(_common.FlyteIdlEntity):
@@ -221,4 +227,9 @@ class TaskLog(_common.FlyteIdlEntity):
         :param flyteidl.core.execution_pb2.TaskLog p:
         :rtype: TaskLog
         """
-        return cls(uri=p.uri, name=p.name, message_format=p.message_format, ttl=p.ttl.ToTimedelta(),)
+        return cls(
+            uri=p.uri,
+            name=p.name,
+            message_format=p.message_format,
+            ttl=p.ttl.ToTimedelta(),
+        )
