@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import flyteidl.admin.execution_pb2 as _execution_pb2
 import flyteidl.admin.node_execution_pb2 as _node_execution_pb2
 import flyteidl.admin.task_execution_pb2 as _task_execution_pb2
@@ -64,7 +62,11 @@ class ExecutionMetadata(_common_models.FlyteIdlEntity):
         :param flyteidl.admin.execution_pb2.ExecutionMetadata pb2_object:
         :return: ExecutionMetadata
         """
-        return cls(mode=pb2_object.mode, principal=pb2_object.principal, nesting=pb2_object.nesting,)
+        return cls(
+            mode=pb2_object.mode,
+            principal=pb2_object.principal,
+            nesting=pb2_object.nesting,
+        )
 
 
 class ExecutionSpec(_common_models.FlyteIdlEntity):
@@ -77,6 +79,7 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         labels=None,
         annotations=None,
         auth_role=None,
+        max_parallelism=None,
     ):
         """
         :param flytekit.models.core.identifier.Identifier launch_plan: Launch plan unique identifier to execute
@@ -86,6 +89,9 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         :param flytekit.models.common.Labels labels: Labels to apply to the execution.
         :param flytekit.models.common.Annotations annotations: Annotations to apply to the execution
         :param flytekit.models.common.AuthRole auth_role: The authorization method with which to execute the workflow.
+        :param max_parallelism int: Controls the maximum number of tasknodes that can be run in parallel for the entire
+            workflow. This is useful to achieve fairness. Note: MapTasks are regarded as one unit, and
+            parallelism/concurrency of MapTasks is independent from this.
 
         """
         self._launch_plan = launch_plan
@@ -95,6 +101,7 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         self._labels = labels or _common_models.Labels({})
         self._annotations = annotations or _common_models.Annotations({})
         self._auth_role = auth_role or _common_models.AuthRole()
+        self._max_parallelism = max_parallelism
 
     @property
     def launch_plan(self):
@@ -146,6 +153,10 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
         """
         return self._auth_role
 
+    @property
+    def max_parallelism(self) -> int:
+        return self._max_parallelism
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.admin.execution_pb2.ExecutionSpec
@@ -158,6 +169,7 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
             labels=self.labels.to_flyte_idl(),
             annotations=self.annotations.to_flyte_idl(),
             auth_role=self._auth_role.to_flyte_idl() if self.auth_role else None,
+            max_parallelism=self.max_parallelism,
         )
 
     @classmethod
@@ -174,6 +186,7 @@ class ExecutionSpec(_common_models.FlyteIdlEntity):
             labels=_common_models.Labels.from_flyte_idl(p.labels),
             annotations=_common_models.Annotations.from_flyte_idl(p.annotations),
             auth_role=_common_models.AuthRole.from_flyte_idl(p.auth_role),
+            max_parallelism=p.max_parallelism,
         )
 
 
@@ -205,7 +218,8 @@ class LiteralMapBlob(_common_models.FlyteIdlEntity):
         :rtype: flyteidl.admin.execution_pb2.LiteralMapBlob
         """
         return _execution_pb2.LiteralMapBlob(
-            values=self.values.to_flyte_idl() if self.values is not None else None, uri=self.uri,
+            values=self.values.to_flyte_idl() if self.values is not None else None,
+            uri=self.uri,
         )
 
     @classmethod
@@ -258,7 +272,9 @@ class Execution(_common_models.FlyteIdlEntity):
         :rtype: flyteidl.admin.execution_pb2.Execution
         """
         return _execution_pb2.Execution(
-            id=self.id.to_flyte_idl(), closure=self.closure.to_flyte_idl(), spec=self.spec.to_flyte_idl(),
+            id=self.id.to_flyte_idl(),
+            closure=self.closure.to_flyte_idl(),
+            spec=self.spec.to_flyte_idl(),
         )
 
     @classmethod
