@@ -1,4 +1,4 @@
-from __future__ import absolute_import
+import typing
 
 import six as _six
 from flyteidl.core import interface_pb2 as _interface_pb2
@@ -47,7 +47,10 @@ class Variable(_common.FlyteIdlEntity):
         :param flyteidl.core.interface_pb2.Variable variable_proto:
         :rtype: Variable
         """
-        return cls(type=_types.LiteralType.from_flyte_idl(variable_proto.type), description=variable_proto.description,)
+        return cls(
+            type=_types.LiteralType.from_flyte_idl(variable_proto.type),
+            description=variable_proto.description,
+        )
 
 
 class VariableMap(_common.FlyteIdlEntity):
@@ -94,23 +97,14 @@ class TypedInterface(_common.FlyteIdlEntity):
         self._outputs = outputs
 
     @property
-    def inputs(self):
-        """
-        :rtype: dict[Text, Variable]
-        """
+    def inputs(self) -> typing.Dict[str, Variable]:
         return self._inputs
 
     @property
-    def outputs(self):
-        """
-        :rtype: dict[Text, Variable]
-        """
+    def outputs(self) -> typing.Dict[str, Variable]:
         return self._outputs
 
-    def to_flyte_idl(self):
-        """
-        :rtype: flyteidl.core.interface_pb2.TypedInterface
-        """
+    def to_flyte_idl(self) -> _interface_pb2.TypedInterface:
         return _interface_pb2.TypedInterface(
             inputs=_interface_pb2.VariableMap(variables={k: v.to_flyte_idl() for k, v in _six.iteritems(self.inputs)}),
             outputs=_interface_pb2.VariableMap(
@@ -119,10 +113,9 @@ class TypedInterface(_common.FlyteIdlEntity):
         )
 
     @classmethod
-    def from_flyte_idl(cls, proto):
+    def from_flyte_idl(cls, proto: _interface_pb2.TypedInterface) -> "TypedInterface":
         """
-        :param flyteidl.core.interface_pb2.TypedInterface proto:
-        :rtype: TypedInterface
+        :param proto:
         """
         return cls(
             inputs={k: Variable.from_flyte_idl(v) for k, v in _six.iteritems(proto.inputs.variables)},
@@ -161,7 +154,7 @@ class Parameter(_common.FlyteIdlEntity):
         return self._default
 
     @property
-    def required(self):
+    def required(self) -> bool:
         """
         If True, this parameter must be specified.  There cannot be a default value.
         :rtype: bool
@@ -182,7 +175,7 @@ class Parameter(_common.FlyteIdlEntity):
         return _interface_pb2.Parameter(
             var=self.var.to_flyte_idl(),
             default=self.default.to_flyte_idl() if self.default is not None else None,
-            required=self.required,
+            required=self.required if self.default is None else None,
         )
 
     @classmethod
