@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 from typing import List
 
 from flyteidl.plugins.sagemaker import training_job_pb2 as _training_job_pb2
@@ -7,6 +5,16 @@ from flyteidl.plugins.sagemaker import training_job_pb2 as _training_job_pb2
 from flytekit.models import common as _common
 
 
+class DistributedProtocol(object):
+    """
+    The distribution framework is used for determining which underlying distributed training mechanism to use.
+    This is only required for use cases where the user wants to train its custom training job in a distributed manner
+    """
+
+    UNSPECIFIED = _training_job_pb2.DistributedProtocol.UNSPECIFIED
+    MPI = _training_job_pb2.DistributedProtocol.MPI
+
+
 class TrainingJobResourceConfig(_common.FlyteIdlEntity):
     """
     TrainingJobResourceConfig is a pass-through, specifying the instance type to use for the training job, the
@@ -15,11 +23,16 @@ class TrainingJobResourceConfig(_common.FlyteIdlEntity):
     """
 
     def __init__(
-        self, instance_count: int, instance_type: str, volume_size_in_gb: int,
+        self,
+        instance_type: str,
+        volume_size_in_gb: int,
+        instance_count: int = 1,
+        distributed_protocol: int = DistributedProtocol.UNSPECIFIED,
     ):
         self._instance_count = instance_count
         self._instance_type = instance_type
         self._volume_size_in_gb = volume_size_in_gb
+        self._distributed_protocol = distributed_protocol
 
     @property
     def instance_count(self) -> int:
@@ -45,6 +58,15 @@ class TrainingJobResourceConfig(_common.FlyteIdlEntity):
         """
         return self._volume_size_in_gb
 
+    @property
+    def distributed_protocol(self) -> int:
+        """
+        The distribution framework is used to determine through which mechanism the distributed training is done.
+        enum value from DistributionFramework.
+        :rtype: int
+        """
+        return self._distributed_protocol
+
     def to_flyte_idl(self) -> _training_job_pb2.TrainingJobResourceConfig:
         """
 
@@ -54,6 +76,7 @@ class TrainingJobResourceConfig(_common.FlyteIdlEntity):
             instance_count=self.instance_count,
             instance_type=self.instance_type,
             volume_size_in_gb=self.volume_size_in_gb,
+            distributed_protocol=self.distributed_protocol,
         )
 
     @classmethod
@@ -67,12 +90,15 @@ class TrainingJobResourceConfig(_common.FlyteIdlEntity):
             instance_count=pb2_object.instance_count,
             instance_type=pb2_object.instance_type,
             volume_size_in_gb=pb2_object.volume_size_in_gb,
+            distributed_protocol=pb2_object.distributed_protocol,
         )
 
 
 class MetricDefinition(_common.FlyteIdlEntity):
     def __init__(
-        self, name: str, regex: str,
+        self,
+        name: str,
+        regex: str,
     ):
         self._name = name
         self._regex = regex
@@ -99,7 +125,10 @@ class MetricDefinition(_common.FlyteIdlEntity):
 
         :rtype: _training_job_pb2.MetricDefinition
         """
-        return _training_job_pb2.MetricDefinition(name=self.name, regex=self.regex,)
+        return _training_job_pb2.MetricDefinition(
+            name=self.name,
+            regex=self.regex,
+        )
 
     @classmethod
     def from_flyte_idl(cls, pb2_object: _training_job_pb2.MetricDefinition):
@@ -108,9 +137,13 @@ class MetricDefinition(_common.FlyteIdlEntity):
         :param pb2_object: _training_job_pb2.MetricDefinition
         :rtype: MetricDefinition
         """
-        return cls(name=pb2_object.name, regex=pb2_object.regex,)
+        return cls(
+            name=pb2_object.name,
+            regex=pb2_object.regex,
+        )
 
 
+# TODO Convert to Enum
 class InputMode(object):
     """
     When using FILE input mode, different SageMaker built-in algorithms require different file types of input data
@@ -122,6 +155,7 @@ class InputMode(object):
     FILE = _training_job_pb2.InputMode.FILE
 
 
+# TODO Convert to enum
 class AlgorithmName(object):
     """
     The algorithm name is used for deciding which pre-built image to point to.
@@ -134,6 +168,7 @@ class AlgorithmName(object):
     XGBOOST = _training_job_pb2.AlgorithmName.XGBOOST
 
 
+# TODO convert to enum
 class InputContentType(object):
     """
     Specifies the type of content for input data. Different SageMaker built-in algorithms require different content types of input data
@@ -157,9 +192,9 @@ class AlgorithmSpecification(_common.FlyteIdlEntity):
 
     def __init__(
         self,
-        algorithm_name: int,
-        algorithm_version: str,
-        input_mode: int,
+        algorithm_name: int = AlgorithmName.CUSTOM,
+        algorithm_version: str = "",
+        input_mode: int = InputMode.FILE,
         metric_definitions: List[MetricDefinition] = None,
         input_content_type: int = InputContentType.TEXT_CSV,
     ):
@@ -243,7 +278,9 @@ class AlgorithmSpecification(_common.FlyteIdlEntity):
 
 class TrainingJob(_common.FlyteIdlEntity):
     def __init__(
-        self, algorithm_specification: AlgorithmSpecification, training_job_resource_config: TrainingJobResourceConfig,
+        self,
+        algorithm_specification: AlgorithmSpecification,
+        training_job_resource_config: TrainingJobResourceConfig,
     ):
         self._algorithm_specification = algorithm_specification
         self._training_job_resource_config = training_job_resource_config
@@ -270,8 +307,12 @@ class TrainingJob(_common.FlyteIdlEntity):
         """
 
         return _training_job_pb2.TrainingJob(
-            algorithm_specification=self.algorithm_specification.to_flyte_idl(),
-            training_job_resource_config=self.training_job_resource_config.to_flyte_idl(),
+            algorithm_specification=self.algorithm_specification.to_flyte_idl()
+            if self.algorithm_specification
+            else None,
+            training_job_resource_config=self.training_job_resource_config.to_flyte_idl()
+            if self.training_job_resource_config
+            else None,
         )
 
     @classmethod
