@@ -1,6 +1,5 @@
-from __future__ import absolute_import
-
 import json as _json
+import typing
 
 import six as _six
 from flyteidl.admin import task_pb2 as _admin_task
@@ -9,6 +8,7 @@ from flyteidl.core import literals_pb2 as _literals_pb2
 from flyteidl.core import tasks_pb2 as _core_task
 from flyteidl.plugins import pytorch_pb2 as _pytorch_task
 from flyteidl.plugins import spark_pb2 as _spark_task
+from flyteidl.plugins import tensorflow_pb2 as _tensorflow_task
 from google.protobuf import json_format as _json_format
 from google.protobuf import struct_pb2 as _struct
 
@@ -16,6 +16,7 @@ from flytekit.common.exceptions import user as _user_exceptions
 from flytekit.models import common as _common
 from flytekit.models import interface as _interface
 from flytekit.models import literals as _literals
+from flytekit.models import security as _sec
 from flytekit.models.core import identifier as _identifier
 from flytekit.plugins import flyteidl as _lazy_flyteidl
 from flytekit.sdk.spark_types import SparkType as _spark_type
@@ -99,7 +100,8 @@ class Resources(_common.FlyteIdlEntity):
         :rtype: flyteidl.core.tasks_pb2.Resources
         """
         return _core_task.Resources(
-            requests=[r.to_flyte_idl() for r in self.requests], limits=[r.to_flyte_idl() for r in self.limits],
+            requests=[r.to_flyte_idl() for r in self.requests],
+            limits=[r.to_flyte_idl() for r in self.limits],
         )
 
     @classmethod
@@ -171,7 +173,14 @@ class RuntimeMetadata(_common.FlyteIdlEntity):
 
 class TaskMetadata(_common.FlyteIdlEntity):
     def __init__(
-        self, discoverable, runtime, timeout, retries, interruptible, discovery_version, deprecated_error_message,
+        self,
+        discoverable,
+        runtime,
+        timeout,
+        retries,
+        interruptible,
+        discovery_version,
+        deprecated_error_message,
     ):
         """
         Information needed at runtime to determine behavior such as whether or not outputs are discoverable, timeouts,
@@ -269,7 +278,8 @@ class TaskMetadata(_common.FlyteIdlEntity):
             discovery_version=self.discovery_version,
             deprecated_error_message=self.deprecated_error_message,
         )
-        tm.timeout.FromTimedelta(self.timeout)
+        if self.timeout:
+            tm.timeout.FromTimedelta(self.timeout)
         return tm
 
     @classmethod
@@ -290,7 +300,19 @@ class TaskMetadata(_common.FlyteIdlEntity):
 
 
 class TaskTemplate(_common.FlyteIdlEntity):
-    def __init__(self, id, type, metadata, interface, custom, container=None):
+    def __init__(
+        self,
+        id,
+        type,
+        metadata,
+        interface,
+        custom,
+        container=None,
+        task_type_version=0,
+        security_context=None,
+        config=None,
+        k8s_pod=None,
+    ):
         """
         A task template represents the full set of information necessary to perform a unit of work in the Flyte system.
         It contains the metadata about what inputs and outputs are consumed or produced.  It also contains the metadata
@@ -305,13 +327,26 @@ class TaskTemplate(_common.FlyteIdlEntity):
         :param dict[Text, T] custom: Dictionary that must be serializable to a protobuf Struct for custom task plugins.
         :param Container container: Provides the necessary entrypoint information for execution.  For instance,
             a Container might be specified with the necessary command line arguments.
+        :param int task_type_version: Specific version of this task type used by plugins to potentially modify
+            execution behavior or serialization.
+        :param dict[str, str] config: For plugin tasks this represents additional configuration information to be used
+            in tandem with the custom.
+        :param dict[str, str] config: For plugin tasks this represents additional configuration information to be used
+            in tandem with the custom.
+        :param K8sPod k8s_pod: Alternative to the container used to execute this task.
         """
+        if container is not None and k8s_pod is not None:
+            raise ValueError("At most one of container or k8s_pod can be set")
         self._id = id
         self._type = type
         self._metadata = metadata
         self._interface = interface
         self._custom = custom
         self._container = container
+        self._task_type_version = task_type_version
+        self._config = config
+        self._security_context = security_context
+        self._k8s_pod = k8s_pod
 
     @property
     def id(self):
@@ -342,7 +377,7 @@ class TaskTemplate(_common.FlyteIdlEntity):
     def interface(self):
         """
         The interface definition for this task.
-        :rtype: flytekit.common.interface.TypedInterface
+        :rtype: flytekit.models.interface.TypedInterface
         """
         return self._interface
 
@@ -354,6 +389,10 @@ class TaskTemplate(_common.FlyteIdlEntity):
         """
         return self._custom
 
+    @property
+    def task_type_version(self):
+        return self._task_type_version
+
     @property
     def container(self):
         """
@@ -362,18 +401,39 @@ class TaskTemplate(_common.FlyteIdlEntity):
         """
         return self._container
 
+    @property
+    def config(self):
+        """
+        Arbitrary dictionary containing metadata for parsing and handling custom plugins.
+        :rtype: dict[Text, T]
+        """
+        return self._config
+
+    @property
+    def security_context(self):
+        return self._security_context
+
+    @property
+    def k8s_pod(self):
+        return self._k8s_pod
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.core.tasks_pb2.TaskTemplate
         """
-        return _core_task.TaskTemplate(
+        task_template = _core_task.TaskTemplate(
             id=self.id.to_flyte_idl(),
             type=self.type,
             metadata=self.metadata.to_flyte_idl(),
             interface=self.interface.to_flyte_idl(),
             custom=_json_format.Parse(_json.dumps(self.custom), _struct.Struct()) if self.custom else None,
             container=self.container.to_flyte_idl() if self.container else None,
+            task_type_version=self.task_type_version,
+            security_context=self.security_context.to_flyte_idl() if self.security_context else None,
+            config={k: v for k, v in self.config.items()} if self.config is not None else None,
+            k8s_pod=self.k8s_pod.to_flyte_idl() if self.k8s_pod else None,
         )
+        return task_template
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -388,6 +448,12 @@ class TaskTemplate(_common.FlyteIdlEntity):
             interface=_interface.TypedInterface.from_flyte_idl(pb2_object.interface),
             custom=_json_format.MessageToDict(pb2_object.custom) if pb2_object else None,
             container=Container.from_flyte_idl(pb2_object.container) if pb2_object.HasField("container") else None,
+            task_type_version=pb2_object.task_type_version,
+            security_context=_sec.SecurityContext.from_flyte_idl(pb2_object.security_context)
+            if pb2_object.security_context
+            else None,
+            config={k: v for k, v in pb2_object.config.items()} if pb2_object.config is not None else None,
+            k8s_pod=K8sPod.from_flyte_idl(pb2_object.k8s_pod) if pb2_object.HasField("k8s_pod") else None,
         )
 
 
@@ -449,7 +515,10 @@ class Task(_common.FlyteIdlEntity):
         """
         :rtype: flyteidl.admin.task_pb2.Task
         """
-        return _admin_task.Task(closure=self.closure.to_flyte_idl(), id=self.id.to_flyte_idl(),)
+        return _admin_task.Task(
+            closure=self.closure.to_flyte_idl(),
+            id=self.id.to_flyte_idl(),
+        )
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
@@ -523,7 +592,13 @@ class CompiledTask(_common.FlyteIdlEntity):
 
 class SparkJob(_common.FlyteIdlEntity):
     def __init__(
-        self, spark_type, application_file, main_class, spark_conf, hadoop_conf, executor_path,
+        self,
+        spark_type,
+        application_file,
+        main_class,
+        spark_conf,
+        hadoop_conf,
+        executor_path,
     ):
         """
         This defines a SparkJob target.  It will execute the appropriate SparkJob.
@@ -539,6 +614,24 @@ class SparkJob(_common.FlyteIdlEntity):
         self._spark_conf = spark_conf
         self._hadoop_conf = hadoop_conf
 
+    def with_overrides(
+        self, new_spark_conf: typing.Dict[str, str] = None, new_hadoop_conf: typing.Dict[str, str] = None
+    ) -> "SparkJob":
+        if not new_spark_conf:
+            new_spark_conf = self.spark_conf
+
+        if not new_hadoop_conf:
+            new_hadoop_conf = self.hadoop_conf
+
+        return SparkJob(
+            spark_type=self.spark_type,
+            application_file=self.application_file,
+            main_class=self.main_class,
+            spark_conf=new_spark_conf,
+            hadoop_conf=new_hadoop_conf,
+            executor_path=self.executor_path,
+        )
+
     @property
     def main_class(self):
         """
@@ -574,8 +667,8 @@ class SparkJob(_common.FlyteIdlEntity):
     @property
     def spark_conf(self):
         """
-       A definition of key-value pairs for spark config for the job.
-        :rtype: dict[Text, Text]
+        A definition of key-value pairs for spark config for the job.
+         :rtype: dict[Text, Text]
         """
         return self._spark_conf
 
@@ -665,7 +758,10 @@ class IOStrategy(_common.FlyteIdlEntity):
     def from_flyte_idl(cls, pb2_object: _core_task.IOStrategy):
         if pb2_object is None:
             return None
-        return cls(download_mode=pb2_object.download_mode, upload_mode=pb2_object.upload_mode,)
+        return cls(
+            download_mode=pb2_object.download_mode,
+            upload_mode=pb2_object.upload_mode,
+        )
 
 
 class DataLoadingConfig(_common.FlyteIdlEntity):
@@ -702,8 +798,7 @@ class DataLoadingConfig(_common.FlyteIdlEntity):
         )
 
     @classmethod
-    def from_flyte_idl(cls, pb2: _core_task.DataLoadingConfig):
-        # TODO use python 3.7+ only and then https://stackoverflow.com/questions/33533148/how-do-i-specify-that-the-return-type-of-a-method-is-the-same-as-the-class-itsel -> DataLoadingConfig:
+    def from_flyte_idl(cls, pb2: _core_task.DataLoadingConfig) -> "DataLoadingConfig":
         if pb2 is None:
             return None
         return cls(
@@ -718,17 +813,17 @@ class DataLoadingConfig(_common.FlyteIdlEntity):
 class Container(_common.FlyteIdlEntity):
     def __init__(self, image, command, args, resources, env, config, data_loading_config=None):
         """
-           This defines a container target.  It will execute the appropriate command line on the appropriate image with
-           the given configurations.
+        This defines a container target.  It will execute the appropriate command line on the appropriate image with
+        the given configurations.
 
-           :param Text image: The fully-qualified identifier for the image.
-           :param list[Text] command: A list of 'words' for the command.  i.e. ['aws', 's3', 'ls']
-           :param list[Text] args: A list of arguments for the command.  i.e. ['s3://some/path', '/tmp/local/path']
-           :param Resources resources: A definition of requisite compute resources.
-           :param dict[Text, Text] env: A definition of key-value pairs for environment variables.
-           :param dict[Text, Text] config: A definition of configuration key-value pairs.
-           :type DataLoadingConfig data_loading_config: object
-           """
+        :param Text image: The fully-qualified identifier for the image.
+        :param list[Text] command: A list of 'words' for the command.  i.e. ['aws', 's3', 'ls']
+        :param list[Text] args: A list of arguments for the command.  i.e. ['s3://some/path', '/tmp/local/path']
+        :param Resources resources: A definition of requisite compute resources.
+        :param dict[Text, Text] env: A definition of key-value pairs for environment variables.
+        :param dict[Text, Text] config: A definition of configuration key-value pairs.
+        :type DataLoadingConfig data_loading_config: object
+        """
         self._data_loading_config = data_loading_config
         self._image = image
         self._command = command
@@ -827,17 +922,83 @@ class Container(_common.FlyteIdlEntity):
         )
 
 
+class K8sObjectMetadata(_common.FlyteIdlEntity):
+    def __init__(self, labels: typing.Dict[str, str] = None, annotations: typing.Dict[str, str] = None):
+        """
+        This defines additional metadata for building a kubernetes pod.
+        """
+        self._labels = labels
+        self._annotations = annotations
+
+    @property
+    def labels(self) -> typing.Dict[str, str]:
+        return self._labels
+
+    @property
+    def annotations(self) -> typing.Dict[str, str]:
+        return self._annotations
+
+    def to_flyte_idl(self) -> _core_task.K8sObjectMetadata:
+        return _core_task.K8sObjectMetadata(
+            labels={k: v for k, v in self.labels.items()} if self.labels is not None else None,
+            annotations={k: v for k, v in self.annotations.items()} if self.annotations is not None else None,
+        )
+
+    @classmethod
+    def from_flyte_idl(cls, pb2_object: _core_task.K8sObjectMetadata):
+        return cls(
+            labels={k: v for k, v in pb2_object.labels.items()} if pb2_object.labels is not None else None,
+            annotations={k: v for k, v in pb2_object.annotations.items()}
+            if pb2_object.annotations is not None
+            else None,
+        )
+
+
+class K8sPod(_common.FlyteIdlEntity):
+    def __init__(self, metadata: K8sObjectMetadata = None, pod_spec: typing.Dict[str, typing.Any] = None):
+        """
+        This defines a kubernetes pod target.  It will build the pod target during task execution
+        """
+        self._metadata = metadata
+        self._pod_spec = pod_spec
+
+    @property
+    def metadata(self) -> K8sObjectMetadata:
+        return self._metadata
+
+    @property
+    def pod_spec(self) -> typing.Dict[str, typing.Any]:
+        return self._pod_spec
+
+    def to_flyte_idl(self) -> _core_task.K8sPod:
+        return _core_task.K8sPod(
+            metadata=self._metadata.to_flyte_idl(),
+            pod_spec=_json_format.Parse(_json.dumps(self.pod_spec), _struct.Struct()) if self.pod_spec else None,
+        )
+
+    @classmethod
+    def from_flyte_idl(cls, pb2_object: _core_task.K8sPod):
+        return cls(
+            metadata=K8sObjectMetadata.from_flyte_idl(pb2_object.metadata),
+            pod_spec=_json_format.MessageToDict(pb2_object.pod_spec) if pb2_object.HasField("pod_spec") else None,
+        )
+
+
 class SidecarJob(_common.FlyteIdlEntity):
-    def __init__(self, pod_spec, primary_container_name):
+    def __init__(self, pod_spec, primary_container_name, annotations=None, labels=None):
         """
         A sidecar job represents the full kubernetes pod spec and related metadata required for executing a sidecar
         task.
 
         :param pod_spec: k8s.io.api.core.v1.PodSpec
         :param primary_container_name: Text
+        :param dict[Text, Text] annotations:
+        :param dict[Text, Text] labels:
         """
         self._pod_spec = pod_spec
         self._primary_container_name = primary_container_name
+        self._annotations = annotations
+        self._labels = labels
 
     @property
     def pod_spec(self):
@@ -853,12 +1014,29 @@ class SidecarJob(_common.FlyteIdlEntity):
         """
         return self._primary_container_name
 
+    @property
+    def annotations(self):
+        """
+        :rtype: dict[Text,Text]
+        """
+        return self._annotations
+
+    @property
+    def labels(self):
+        """
+        :rtype: dict[Text,Text]
+        """
+        return self._labels
+
     def to_flyte_idl(self):
         """
         :rtype: flyteidl.core.tasks_pb2.SidecarJob
         """
         return _lazy_flyteidl.plugins.sidecar_pb2.SidecarJob(
-            pod_spec=self.pod_spec, primary_container_name=self.primary_container_name
+            pod_spec=self.pod_spec,
+            primary_container_name=self.primary_container_name,
+            annotations=self.annotations,
+            labels=self.labels,
         )
 
     @classmethod
@@ -867,7 +1045,12 @@ class SidecarJob(_common.FlyteIdlEntity):
         :param flyteidl.admin.task_pb2.Task pb2_object:
         :rtype: Container
         """
-        return cls(pod_spec=pb2_object.pod_spec, primary_container_name=pb2_object.primary_container_name,)
+        return cls(
+            pod_spec=pb2_object.pod_spec,
+            primary_container_name=pb2_object.primary_container_name,
+            annotations=pb2_object.annotations,
+            labels=pb2_object.labels,
+        )
 
 
 class PyTorchJob(_common.FlyteIdlEntity):
@@ -879,8 +1062,44 @@ class PyTorchJob(_common.FlyteIdlEntity):
         return self._workers_count
 
     def to_flyte_idl(self):
-        return _pytorch_task.DistributedPyTorchTrainingTask(workers=self.workers_count,)
+        return _pytorch_task.DistributedPyTorchTrainingTask(
+            workers=self.workers_count,
+        )
 
     @classmethod
     def from_flyte_idl(cls, pb2_object):
-        return cls(workers_count=pb2_object.workers,)
+        return cls(
+            workers_count=pb2_object.workers,
+        )
+
+
+class TensorFlowJob(_common.FlyteIdlEntity):
+    def __init__(self, workers_count, ps_replicas_count, chief_replicas_count):
+        self._workers_count = workers_count
+        self._ps_replicas_count = ps_replicas_count
+        self._chief_replicas_count = chief_replicas_count
+
+    @property
+    def workers_count(self):
+        return self._workers_count
+
+    @property
+    def ps_replicas_count(self):
+        return self._ps_replicas_count
+
+    @property
+    def chief_replicas_count(self):
+        return self._chief_replicas_count
+
+    def to_flyte_idl(self):
+        return _tensorflow_task.DistributedTensorflowTrainingTask(
+            workers=self.workers_count, ps_replicas=self.ps_replicas_count, chief_replicas=self.chief_replicas_count
+        )
+
+    @classmethod
+    def from_flyte_idl(cls, pb2_object):
+        return cls(
+            workers_count=pb2_object.workers,
+            ps_replicas_count=pb2_object.ps_replicas,
+            chief_replicas_count=pb2_object.chief_replicas,
+        )
